//-----------------------------------------------------------------------------
// File: PixelMotionBlur.fx
//
// Desc: Effect file for image based motion blur. The HLSL shaders are used to
//       calculate the velocity of each pixel based on the last frame's matrix 
//       transforms.  This per-pixel velocity is then used in a blur filter to 
//       create the motion blur effect.
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4 MaterialAmbientColor;
float4 MaterialDiffuseColor;

float3 LightDir = normalize(float3(1.0f, 1.0f, 1.0f));
float4 LightAmbient = { 1.0f, 1.0f, 1.0f, 1.0f };    // ambient
float4 LightDiffuse = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse

float DistortionPosx ;//= (float2(0.0f, 0.0f));
float DistortionPosy ;

texture RenderTargetTexture;
texture RenderTargetTexture2;
texture MeshTexture;
texture CurFrameVelocityTexture;
texture LastFrameVelocityTexture;

float4x4 mWorld;
float4x4 mWorldViewProjection;
float4x4 mWorldViewProjectionLast;

float PixelBlurConst = 1.0f;
static const float NumberOfPostProcessSamples = 12.0f;
float ConvertToNonHomogeneous;
float VelocityCapSq = 1.0f;
float RenderTargetWidth;
float RenderTargetHeight;

float MagicValue;;

float2 MagicArray[100];

float4 LightPosition;
float4 CamerPosition;
float AnimationTimer;
float vertexin;
//float MagicArray2[10000];
//-----------------------------------------------------------------------------
// Texture samplers
//-----------------------------------------------------------------------------
sampler RenderTargetSampler = 
sampler_state
{
    Texture = <RenderTargetTexture>;
    MinFilter = LINEAR;  
    MagFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};
sampler RenderTargetSampler2 = 
sampler_state
{
    Texture = <RenderTargetTexture2>;
    MinFilter = POINT;  
    MagFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};
sampler CurFramePixelVelSampler = 
sampler_state
{
    Texture = <CurFrameVelocityTexture>;
    MinFilter = POINT;
    MagFilter = POINT;

    AddressU = Clamp;
    AddressV = Clamp;
};






//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position : POSITION;   // position of the vertex
    float4 Diffuse  : COLOR0;     // diffuse color of the vertex
    float2 TextureUV : TEXCOORD0;  // typical texture coords stored here
   // float2 VelocityUV : TEXCOORD1;  // per-vertex velocity stored here
};
struct VS_OUTPUT2
{
    float4 Position : POSITION;   // position of the vertex
    float4 Diffuse  : COLOR0;     // diffuse color of the vertex
    float4 PosMagic  : TEXCOORD1;     // diffuse color of the vertex
    float4 NormalMagic : TEXCOORD2;
    float2 TextureUV : TEXCOORD0;  // typical texture coords stored here
   // float2 VelocityUV : TEXCOORD1;  // per-vertex velocity stored here
};

//-----------------------------------------------------------------------------
// Name: WorldVertexShader     
// Type: Vertex shader                                      
// Desc: In addition to standard transform and lighting, it calculates the velocity 
//       of the vertex and outputs this as a texture coord.
//-----------------------------------------------------------------------------
VS_OUTPUT WorldVertexShader( float3 vPos : POSITION, 
                             float4 Diffuse  : COLOR0,
                             float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT Output;


    // Output the vetrex position in projection space
   // vPos.y=10;
    //vPos.x=vPos.x*2.0f;
    //vPos = vPos* 1.000010f;//*length(Output.Position);
    //vPos.w=1.0f;
    Output.Position = mul(float4(vPos,1.0f),mWorldViewProjection);


	
//vertexin=1.0f;//vPos.x;
//Output.Position.z=1.0f;
//Output.Position=1.0f;
    // Store the velocity in a texture coord
    //Output.VelocityUV = 1.0f;
        
    // Compute simple lighting equation
    Output.Diffuse = Diffuse;///(0.001*Output.Position.z*Output.Position.z);                
    Output.Diffuse.a = (0.001*Output.Position.z*Output.Position.z);   
    // Just copy the texture coordinate through
    Output.TextureUV = vTexCoord0; 
    
    return Output;    
}


//-----------------------------------------------------------------------------
// Name: ShadowPlaneVertexShader    
 
VS_OUTPUT ShadowPlaneVertexShader( float3 vPos : POSITION,
									float3 vNormal : NORMAL,
                             float4 Diffuse  : COLOR0,
                             float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT Output;


	float3 LightDir=float4(vPos,1)-LightPosition;

LightDir=normalize(LightDir);
	float DistanceFromPlane=vPos.y+1.35f;
	if (LightDir.y>-0.50f)
	{
	LightDir.y=-0.5f;
	}
	float Muliplier=-DistanceFromPlane/LightDir.y;
	LightDir*=Muliplier;
	//LightDir.y=-vPos.y;
	vPos+=LightDir;
	//vPos.y=0.0f;

    Output.Position = mul(float4(vPos,1.0f),mWorldViewProjection);


    // Compute simple lighting equation
    Output.Diffuse = Diffuse*0.0f;///(0.001*Output.Position.z*Output.Position.z); 
    Output.Diffuse.a=Diffuse.a;               
    //Output.Diffuse.a = (0.001*Output.Position.z*Output.Position.z);   
    // Just copy the texture coordinate through
    Output.TextureUV = vTexCoord0; 
    
    return Output;    
}
           		
 
//-----------------------------------------------------------------------------
// Name: ShadowPlanePS 
// Type: Pixel shader                                      
float4 ShadowPlanePS( float2 OriginalUV : TEXCOORD0 ,float4 Diffuse  : COLOR0) : COLOR
{
       
    int ux=OriginalUV.x;
    OriginalUV.x-=ux;
    int uy=OriginalUV.y;
    OriginalUV.y-=uy;
	float4 Current = tex2D(RenderTargetSampler, OriginalUV);
	
	
		
	//Current-=0.50f;
	Current*=Diffuse;
	Current.a=Diffuse.a;
	//Current.z=1.0f;//max(Current,Other);
	return Current;
}
//-----------------------------------------------------------------------------
// Name: WorldVertexShader     
// Type: Vertex shader                                      
// Desc: In addition to standard transform and lighting, it calculates the velocity 
//       of the vertex and outputs this as a texture coord.
//-----------------------------------------------------------------------------
VS_OUTPUT2 WorldLightingVertexShader( float3 vPos : POSITION,
									float3 vNormal : NORMAL,
                             float4 Diffuse  : COLOR0,
                             float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT2 Output;

    Output.Position = mul(float4(vPos,1.0f),mWorldViewProjection);

	//float4 LightDir=LightPosition-float4(vPos,1);

	//LightDir=normalize(LightDir);

	//float Brightness=dot(vNormal,LightDir);
	//Brightness+=0.5f;
    // Compute simple lighting equation
    Output.Diffuse = Diffuse;              
    //Output.Diffuse.a = (0.001*Output.Position.z*Output.Position.z);   
    // Just copy the texture coordinate through
    Output.TextureUV = vTexCoord0; 

    Output.PosMagic=float4(vPos,0);
    Output.NormalMagic=float4(vNormal,0);
    return Output;    
}

//-----------------------------------------------------------------------------
// Pixel shader output structure
//-----------------------------------------------------------------------------
struct PS_OUTPUT
{
    // The pixel shader can output 2+ values simulatanously if 
    // d3dcaps.NumSimultaneousRTs > 1
    
    float4 RGBColor      : COLOR0;  // Pixel color    
    float4 PixelVelocity : COLOR1;  // Pixel velocity 
};






//-----------------------------------------------------------------------------

// Type: Pixel shader                                      
float4 WolrdLightingPixelShader( float2 OriginalUV : TEXCOORD0 ,float4 Diffuse  : COLOR0,float4 PosMagic  : TEXCOORD1,float4 NormalMagic  : TEXCOORD2) : COLOR
{
       
     int ux=OriginalUV.x;
    OriginalUV.x-=ux;
    int uy=OriginalUV.y;
    OriginalUV.y-=uy;
	float DiffuseAlpha=Diffuse.a;
    
 
	float4 Current = tex2D(RenderTargetSampler, OriginalUV);   
  	float4 LightDir=LightPosition-PosMagic;
	LightDir.a=0.0f;
	float LightDist=length(LightDir);
	//float LightDistOrig=LightDist;
	LightDir=LightDir/LightDist;//normalize(LightDir);
	LightDist*=0.0003f;
	LightDist*=LightDist;

LightDir=normalize(LightDir);
	//look up shadow map value	

	//specular!
	float4 vCamToPixel=CamerPosition-PosMagic;	
	vCamToPixel=normalize(vCamToPixel);
	LightDir=normalize(LightDir);
	
	float4 ReflectDir=-(LightDir-(NormalMagic*2.0f*dot(NormalMagic,LightDir)));
	float CosTheta=dot(ReflectDir,vCamToPixel);
	
	float Brightness=dot(NormalMagic,LightDir)*(1.0f/LightDist);
	

	float Specular=clamp(CosTheta,0.0f,1000.0f);
	Specular*=Specular*Specular*Specular*Specular;//*Specular*Specular*Specular*Specular*Specular*Specular;
	
	if (dot(NormalMagic,vCamToPixel)<0)
	{
		Specular=0.0f;
		//Diffuse.r=1.0f;
		//NormalMagic=-NormalMagic;
		//Diffuse.r=0.0f;
		Brightness=0.0f;
	}	
	//Brightness+=200000.0f;
	//Brightness=max(Brightness,30000.0f);
    // Compute simple lighting equation
    Diffuse = Diffuse*Brightness*0.0000020f;   
    
    
    
	
    Diffuse+=(Specular*0.5f);
    //if (length(Current)<2.0f)
	//{
	//	Diffuse=100.0f;
	//}
   
	Current*=Diffuse;
	Current+=0.05f;
	//Current.b=
	Current.a=DiffuseAlpha;
	Current.b=0.10f;
	
	
	
	//Current.z=1.0f;//max(Current,Other);
	return Current;
}

float4 WolrdShadowMapPixelShader( float2 OriginalUV : TEXCOORD0 ,float4 Diffuse  : COLOR0,float4 PosMagic  : TEXCOORD1,float4 NormalMagic  : TEXCOORD2) : COLOR
{
    

	float4 Current=PosMagic.b;

	Current.a=1.0f;
	Current.b=Current.b*0.01f;
	Current.b+=0.01f;
	Current.r=Current.b;
	Current.g=Current.b;	
	//Current.z=1.0f;//max(Current,Other);
	return Current;
}
float4 PostProcessMotionBlurPS( float2 OriginalUV : TEXCOORD0 ) : COLOR
{    
 
    
    float4 Current = tex2D(RenderTargetSampler, OriginalUV);
	//float4 Current2 = tex2D(RenderTargetSampler2, OriginalUV);	
    //Current.r=(Current.r+0.10f);
     
    //Current.r=Current.r*4.0f*Current.r;
   // Current.g=Current.r;
    //Current.b=Current.r;
    //Current.r=clamp(Current.r,0.0f,1.0f);
    //Current=Current2*Current.r;
    // Return the average color of all the samples
    //return float4(1.0f,1.0f,1.0f,1.0f);//Blurred / NumberOfPostProcessSamples, 1.0f);
    return Current;//float4(Blurred , 1.0f);
}


//-----------------------------------------------------------------------------
// Name: WorldWithVelocityMRT
// Type: Technique                                     
// Desc: Renders the scene's color to render target 0 and simultaneously writes 
//       pixel velocity to render target 1.
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Name: PostProcessMotionBlur
// Type: Technique                                     
// Desc: Renders a full screen quad and uses velocity information stored in 
//       the textures to blur image.
//-----------------------------------------------------------------------------
technique PostProcessMotionBlur
{
    pass P0
    {  
		VertexShader = compile vs_2_0 WorldVertexShader();      
        PixelShader = compile ps_2_0 PostProcessMotionBlurPS();
    }
}

technique WorldLighting
{
    pass P0
    { 
		VertexShader = compile vs_2_0 WorldLightingVertexShader();             		
        PixelShader = compile ps_2_0 WolrdLightingPixelShader();
    }
}
technique RenderToShadowPlane
{
    pass P0
    { 
		VertexShader = compile vs_2_0 ShadowPlaneVertexShader();             		
        PixelShader = compile ps_2_0 ShadowPlanePS();
    }
}
technique RenderToShadowMap
{
    pass P0
    { 
		VertexShader = compile vs_2_0 WorldLightingVertexShader();             		
        PixelShader = compile ps_2_0 WolrdShadowMapPixelShader();
    }
}

